////////////////////////////////////////////////////////////////////////////////
//Description:  KWC Receiver stress tester H-Bridge driver                     /
//Firmware:     0194-00-00                                                     /
//Hardware:                                                                    /
//Processor:    PIC18F2520                                                     /
////////////////////////////////////////////////////////////////////////////////
//Revision history                                                             /
//No.   Date        Description                                                /
//-----------------------------------------------------------------------------/
//00    yyyy/mm/dd  Initial release                                            /
////////////////////////////////////////////////////////////////////////////////
#define     debugme         0           //0=standalone / 1=debug
////////////////////////////////////////////////////////////////////////////////
#include    "019400.h"
#include    "pinmap.h"
////////////////////////////////////////////////////////////////////////////////
#define     mon_freq        32767       //loop prescaler
#define     mon_lamp        b4lat       //use pinmap.h defs (set dir in main)
#include    "mon_flash.c"
//------------------------------------------------------------------------------
#define     scmdstream      pcport      //more than 1 stream, scmd uses spec'd
#define     scmduart        1           //1=rda, 2=rda2
#define     scmdbuffer      19          //default buffer size, limited by ram
byte const  scmd_c1='0';                //sw core, command 00 response data
byte const  scmd_c2='1';
byte const  scmd_c3='9';
byte const  scmd_c4='4';
byte const  scmd_v1='0';                //sw variant
byte const  scmd_v2='0';
byte const  scmd_r1='0';                //sw revision
byte const  scmd_r2='0';
#include    "scmd.c"                    //set path in project
#include    "convert.c"                 //main code, not interrupt handler
//------------------------------------------------------------------------------
#define     lcd_rs          c0lat       //(4)  Control registers
#define     lcd_rw          a5lat       //(5)
#define     lcd_en          c1lat       //(6)
#define     lcd_d0          c2pin       //(11) Data registers
#define     lcd_d1          c3pin       //(12)
#define     lcd_d2          c4pin       //(13)
#define     lcd_d3          c5pin       //(14)
#define     lcd_rs_dir      c0dir       //control direction registers
#define     lcd_rw_dir      a5dir       
#define     lcd_en_dir      c1dir
#define     lcd_d0_dir      c2dir       //data direction registers
#define     lcd_d1_dir      c3dir
#define     lcd_d2_dir      c4dir
#define     lcd_d3_dir      c5dir
#include    "lcd4bit.c"
////////////////////////////////////////////////////////////////////////////////
#include    "019400_memmap.c"
#include    "019400_interrupt.c"
#include    "019400_utility.c"
#include    "019400_protocol.c"
////////////////////////////////////////////////////////////////////////////////













////////////////////////////////////////////////////////////////////////////////
void main(){
unsigned int16  tick_counter=0;             //9766 x 102.4uS = 1S + 38.4uS
unsigned int1   tick;
unsigned int8   br_counter=0;
unsigned int8   bl_counter=0;
unsigned int8   blr_counter=0;
////////////////////////////////////////////////////////////////////////////////
meminit();
scmd_init(0,0);
if (lcd_detect)
    {
    lcd_installed=1;
    lcd_init();
    printf(lcd_char,"SystemUp");
    delay_ms(2000);
    }
else
    lcd_installed=0;
reset_count();
////////////////////////////////////////////////////////////////////////////////
while (true){
restart_wdt();
mon_flash();
protocol();
//..............................................................................
tick=0;
if (systime())
    {
    tick_counter++;
    if (tick_counter>=977)
        {
        tick_counter=0;
        tick=1;
        }
    }
//------------------------------------------------------------------------------
if (start_cycle)
    {
    start_cycle=0;
    delay_ms(50);
    tick_counter=0;
    br_counter=0;
    bl_counter=0;
    pulse_hl_off();
    //flush rs232 and lcd buffers
    while ((lcd_start!=lcd_end) || (rs232_start!=rs232_end))
        {
        restart_wdt();
        systime();
        }
    display_count();
    }

if (tick)
    {
    //..........................................................................
    //both buttons down displays parameters
    if ((!button_right) && (!button_left))
        {
        if (blr_counter<100)
            {
            blr_counter++;
            bl_counter=0;
            br_counter=0;
            if (blr_counter == 50)
                {
                lcd_cmmd(lcd_line1);
                printf(lcd_char,"a +%02U-%02U",t_a_on,t_a_off);
                lcd_cmmd(lcd_line2);
                printf(lcd_char,"x +%02U-%02U",t_b_on,t_b_off);
                }
            if (blr_counter >= 100)
                {
                lcd_cmmd(lcd_line1);
                printf(lcd_char,"Thr:%04LU",stop_threshold);
                lcd_cmmd(lcd_line2);
                printf(lcd_char,"Stp:%04LU",stop_count);
                }
            }
        }
    //..........................................................................
    //left or right only starts or clears current cycle count
    else
        {
        //clear from last param display
        if (blr_counter>49)
            display_count();
        blr_counter=0;
        //clear count by holding down stop in main loop
        if (!button_right)
            {
            br_counter++;
            if (br_counter>=50)
                {
                br_counter=0;
                reset_count();
                }
            }
        else
            br_counter=0;
        //start cycle by holding down start in main loop
        if (!button_left)
            {
            bl_counter++;
            if (bl_counter>=20)
                {
                bl_counter=0;
                start_cycle=1;
                }
            }
        else
            bl_counter=0;
        }
    //..........................................................................
    }

//------------------------------------------------------------------------------
}}
////////////////////////////////////////////////////////////////////////////////
