////////////////////////////////////////////////////////////////////////////////
//protool handler function for calling from main while(true) loop              /
//scmd  [0]     command data length in bytes                                   /
//      [1]     command code                                                   /
//      [2-18]  command data, binary                                           /
////////////////////////////////////////////////////////////////////////////////
void protocol(void){
unsigned int16 temp1;
unsigned int16 temp2;
//------------------------------------------------------------------------------
//overrun error clearing, perform every system loop
//change to xxx1 or xxx2 to match port used
//------------------------------------------------------------------------------
if (oerr1)                              //if overrun is true
    {                                   //  (Setup in scmd.c vs device number)
    cren1=false;                        //toggle rcsta, cren bit to clear,
    cren1=true;                         //  must match uart #
    }
//------------------------------------------------------------------------------
//incomming command handler
//------------------------------------------------------------------------------
if (scmd_ready)
    {
    switch (scmd[1])
        {
        //......................................................................
        case 0x30:
            //report stop threshold and count and timing parameters
            if (scmd[0]==0)
                {
                printf("!0030,%04LX,%04LX,%02X,%02X,%02X,%02X\r\n",stop_threshold,stop_count,t_a_on,t_a_off,t_b_on,t_b_off);
                scmd[0]=0;
                }
            else
                scmd[0]='f';
            break;
        case 0x31:
            //set stop threshold and count and timing parameters
            if (scmd[0]==8)
                {
                disable_interrupts(global);
                //..............................
                //copy to local variables
                //on timers must be 1mS-99mS
                //off timers must be 0ms-99mS
                //threshold from 0mS-9999mS
                //count from 1 to 9999
                //..............................
                temp1=make16(scmd[2],scmd[3]);
                if (temp1 > 9999)
                    {
                    temp1=9999;
                    scmd[2]=make8(temp1,1);
                    scmd[3]=make8(temp1,0);
                    }
                stop_threshold=temp1;
                //..............................
                temp1=make16(scmd[4],scmd[5]);
                if (temp1 > 9999)
                    {
                    temp1=9999;
                    scmd[2]=make8(temp1,1);
                    scmd[3]=make8(temp1,0);
                    }
                if (temp1 == 0)
                    {
                    temp1=1;
                    scmd[4]=0;
                    scmd[5]=1;
                    }
                stop_count=temp1;
                //..............................
                //ab on time
                if (scmd[6]==0)
                    scmd[6]=1;
                if (scmd[6]>99)
                    scmd[6]=99;
                t_a_on=scmd[6];
                //..............................
                //ab off time
                if (scmd[7]>99)
                    scmd[7]=99;
                t_a_off=scmd[7];
                //..............................
                //xy on time
                if (scmd[8]==0)
                    scmd[8]=1;
                if (scmd[8]>99)
                    scmd[8]=99;
                t_b_on=scmd[8];
                //..............................
                //xy off time
                if (scmd[9]>99)
                    scmd[9]=99;
                t_b_off=scmd[9];
                //..............................
                //save to eeprom
                write_eeprom(0x00,scmd[2]);
                write_eeprom(0x01,scmd[3]);
                write_eeprom(0x02,scmd[4]);
                write_eeprom(0x03,scmd[5]);
                write_eeprom(0x04,scmd[6]);
                write_eeprom(0x05,scmd[7]);
                write_eeprom(0x06,scmd[8]);
                write_eeprom(0x07,scmd[9]);
                //restart interrupts
                enable_interrupts(global);
                scmd[0]='a'; 
                }
            else
                scmd[0]='f';
            break;
        //......................................................................    
        case 0x20:
            //report cycle count
            if (scmd[0]==0)
                {
                printf("!0020,%09LU\r\n",cycle_counter);
                scmd[0]=0; 
                }
            else
                scmd[0]='f';
            break;        
        case 0x21:
            //clear cycle count
            if (scmd[0]==0)
                {
                scmd[0]='a'; 
                reset_count();
                }
            else
                scmd[0]='f';
            break;
        //......................................................................
        case 0x10:
            //start test run
            if (scmd[0]==2)   
                {
                test_count=make16(scmd[2],scmd[3]);
                start_cycle=1;
                scmd[0]=0; 
                }
            else
                scmd[0]='f';
            break;
        //......................................................................
        case 0xff:
            //display menu
            if (scmd[0]==0)   
                {
                printf("H-Bridge driver test fixture\r\n");
                printf("HW:0191 FW:0194.00.00\r\n\r\n");
                
                printf("$0010nnnn[cr]                     Start test, run nnnn cycles (Hex)\r\n");
                printf("!0010a[cr]                        (0000 runs continuously)\r\n\r\n"); 
                
                printf("$0020[cr]                         Report cycle count\r\n");
                printf("!0020,nnnnnnnnn[cr]               Cycle count (Decimal)\r\n\r\n");
                
                printf("$0021[cr]                         Clear cycle count\r\n");
                printf("!0021a[cr]\r\n\r\n");
                
                printf("$0030[cr]                         Report test parameters (Hex)\r\n");
                printf("!0030,tttt,cccc,aa,bb,xx,yy[cr]   tttt:  Stop threshold\r\n");
                printf("                                  cccc:  Stop count\r\n");
                printf("                                  aa:    Right leg on,  mS\r\n");
                printf("                                  bb:              off, mS\r\n");
                printf("                                  xx:    Left leg  on,  mS\r\n");
                printf("                                  yy:              off, mS\r\n\r\n");

                printf("$0031ttttccccaabbxxyy[cr]         Set test parameters (Hex)\r\n");
                printf("!0031a[cr]                        tttt:  Stop threshold\r\n");
                printf("                                  cccc:  Stop count\r\n");
                printf("                                  aa:    Right leg on,  mS\r\n");
                printf("                                  bb:              off, mS\r\n");
                printf("                                  xx:    Left leg  on,  mS\r\n");
                printf("                                  yy:              off, mS\r\n\r\n");

                printf("!0040ccccccaaaxxx[cr]             Cycle test stats (Hex)\r\n");
                printf("                                  cccccc Cycle count\r\n");
                printf("                                  aaa    Leg a-b current\r\n");
                printf("                                  xxx    Leg x-y current\r\n\r\n");
                
                printf("  The 'Right leg' cycle is the left high and the right low, i.e. ADC\r\n");
                printf("measuring current at the right side of the bridge to ground and the 'Left\r\n");
                printf("leg' cycle is conversely the right high and the left low:  For the DB9-M,\r\n");
                printf("this is pins 2-/7+ and 2+/7- respectively.  Leg current can be calculated\r\n");
                printf("from: Iout = ((ADC/1023)*2.5)/Rlow.\r\n\r\n");
     
                printf("  The stop threshold is the ADC reading considered to be an open bridge\r\n");
                printf("and the stop count is the number of times (Non-consecutive) the ADC\r\n");
                printf("reading falls below that threshold after which the test is terminated.\r\n");
                printf("The stop threshold can range from 0 to 9999 (1023 to 9999 is STOP!) and\r\n");
                printf("the stop count can range from 1 to 9999.\r\n\r\n");
     
                printf("  The test times are in mS, but are actually 1.024mS because of the way\r\n");
                printf("time is counted in the PIC.  A setting of '10' will result in and actual\r\n");
                printf("time of 10.24mS.  Minimum on time is 1mS, minimum off time is 0mS.  Note\r\n");
                printf("that 5mS is the minimum integration time where current is read at the end\r\n");
                printf("of the on cycle and shorter times will result in inaccurate ADC readings.\r\n");
                printf("Faster integration can be acheived by reducing the value of R9 and R10.\r\n");
                printf("The maximum on and off times are 99mS.\r\n\r\n");
                
                printf("  The left button (Facing the test connector) starts testing when pressed\r\n");
                printf("for 2 or more seconds.  The right button stops a test in progress immediately\r\n");
                printf("and when idle clears the test count after 5 seconds.  Pressing both buttons\r\n");
                printf("displays the timing parameters for 5 seconds and then displays the threshold\r\n");
                printf("and count limits until either button is released.\r\n\r\n");
                
                printf("  During testing, data is transmitted to RS232 and displayed in HEX (to save\r\n");
                printf("time) but after testing has stopped and during parameter display, values are\r\n");
                printf("shown in decimal.  Any RS232 command received during testing will stop the\r\n");
                printf("testing immediately.\r\n\r\n");
                
                printf("DSUB9-M connector pin out:\r\n");
                printf("1   Right-leg High    When not testing, pots may be read at H-to-M and M-to-L\r\n");
                printf("2   Right-leg Middle  for the purpose of setting the bridge current.  Just be\r\n");
                printf("3   Right-leg Low     sure to remove any UUT when setting the pots.\r\n");
                printf("    ----------------\r\n");
                printf("6   Left-leg  High\r\n");
                printf("7   Left-leg  Middle\r\n");
                printf("8   Left-leg  Low\r\n");
                printf("    ----------------\r\n");
                printf("4   Ground\r\n");
                printf("9   Left (Start) button, active low with no ESD protection\r\n");   
                printf("5   Right (Stop/Reset) button, active low with no ESD protection\r\n\r\n");
                
                printf("Capture this menu to file if the terminal scroll back buffer makes it hard\r\n");
                printf("to read the top of the menu!\r\n\r\n");
                scmd[0]=0;
                }
            else
                scmd[0]='f';
            break;
        //......................................................................
        default:
            scmd[0]='?'; 
            break;
        }                      
    if (scmd[0]>0){                    
        putc('!');                   
        bintohex(scmd_address,scmd_byte); 
        putc(scmd_byte[0]);
        putc(scmd_byte[1]);
        bintohex(scmd[1],scmd_byte);
        putc(scmd_byte[0]);
        putc(scmd_byte[1]);
        putc(scmd[0]);              
        putc(0x0d);                 
        putc(0x0a);
        }
    scmd_ready=false;             
    }
//------------------------------------------------------------------------------
}
////////////////////////////////////////////////////////////////////////////////
